"""Configure pytest for the tests in this directory."""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

import pytest

from simple_toml_settings.settings import TOMLSettings

if TYPE_CHECKING:
    from pyfakefs.fake_filesystem import FakeFilesystem


class SettingsExample(TOMLSettings):
    """Define a class for testing the Settings class."""

    test_string_var: str = "test_value"
    test_int_var: int = 42


@pytest.fixture
def settings(fs: FakeFilesystem) -> SettingsExample:
    """Return a Settings object for testing.

    This fixture creates a fake home directory in a virtual filesystem. It then
    creates a Settings object for the test and returns it. This Settings object
    creates a settings file in the fake home directory.
    """
    # Create a fake home directory for the test
    fs.create_dir(Path.home())

    # Create and return a Settings object for the test
    return SettingsExample("test_app")


@pytest.fixture
def flat_settings(fs: FakeFilesystem) -> SettingsExample:
    """Return a Settings object for testing with a flat config."""
    # Create a fake home directory for the test
    fs.create_dir(Path.home())

    # Create and return a Settings object for the test
    return SettingsExample("test_app", flat_config=True)


@pytest.fixture
def xdg_settings(fs: FakeFilesystem) -> SettingsExample:
    """Return a Settings object for testing.

    This fixture creates a fake home directory in a virtual filesystem. It then
    creates a Settings object for the test and returns it. This Settings object
    creates a settings file in the fake home directory.
    """
    # Create a fake home directory for the test
    fs.create_dir(Path.home())

    # Create and return a Settings object for the test
    return SettingsExample("test_app", xdg_config=True)



"""Defines the package's public interface."""

import os
from pathlib import Path

# https://github.com/srstevenson/xdg-base-dirs/blob/main/src/xdg_base_dirs/__init__.py
# Copyright © Scott Stevenson <scott@stevenson.io>
#
# Permission to use, copy, modify, and/or distribute this software for
# any purpose with or without fee is hereby granted, provided that the
# above copyright notice and this permission notice appear in all
# copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
# WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
# AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
# DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
# PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
# TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.


def _path_from_env(variable: str, default: Path) -> Path:
    """Read an environment variable as a path.

    The environment variable with the specified name is read, and its
    value returned as a path. If the environment variable is not set, is
    set to the empty string, or is set to a relative rather than
    absolute path, the default value is returned.

    Parameters
    ----------
    variable : str
        Name of the environment variable.
    default : Path
        Default value.

    Returns:
    -------
    Path
        Value from environment or default.

    """
    if (value := os.environ.get(variable)) and (
        path := Path(value)
    ).is_absolute():
        return path
    return default


def xdg_config_home() -> Path:
    """Return a Path corresponding to XDG_CONFIG_HOME."""
    return _path_from_env("XDG_CONFIG_HOME", Path.home() / ".config")
